const {
    JSDOM
} = require("jsdom");
const fs = require("fs");
const converter = require("./showdown")(2);

const folder = `${__dirname}/../`;
console.log("Refreshing blog sub-sites...");

fs.readFile(`${folder}index.html`, function (_, html) {
    let templateDom = new JSDOM(html);
    // remove all non-blog stuff from the template (which is just our index.html)
    let noBlog = templateDom.window.document.getElementsByClassName("no-blog");
    while (noBlog.length > 0)
        noBlog[0].parentNode.removeChild(noBlog[0]);
    // change all relative links to move to the parent directory (since we're in blog/)
    let elements = templateDom.window.document.getElementsByTagName("*");
    for (let i = 0; i < elements.length; i++) {
        let element = elements[i];
        if (element.src && element.src.startsWith("./"))
            element.src = `../${element.src.substring(2)}`;
        if (element.href && element.href.startsWith("./"))
            element.href = `../${element.href.substring(2)}`;
    }
    const template = templateDom.serialize();

    fs.readFile(`${folder}blog/src/posts.json`, function (_, data) {
        const json = JSON.parse(data.toString());
        for (let i = 0; i < json.length; i++) {
            const post = json[i];
            const index = i;
            const last = getAdjacentPost(json, index, -1);
            const next = getAdjacentPost(json, index, 1);

            let extension = post.book ? "html" : "md";
            fs.readFile(`${folder}blog/src/${post.id}.${extension}`, function (_, file) {
                let dom = new JSDOM(template);
                let document = dom.window.document;

                document.title += ` - ${post.name}`;
                document.querySelector('meta[property="og:title"]').setAttribute("content", post.name);
                document.querySelector('meta[name="description"]').setAttribute("content", post.summary);
                document.querySelector('meta[property="og:description"]').setAttribute("content", post.summary);

                document.getElementById("nav-items").innerHTML = /*html*/ `
                    <a class="nav-item nav-link" href="../?cat=${post.cat[0]}#blog">Back to Main Page</a>
                    ${last ? /*html*/ `<a class="nav-item nav-link" href="./${last.id}">Previous ${post.cat[0]} Post</a>` : ""} 
                    ${next ? /*html*/ `<a class="nav-item nav-link" href="./${next.id}">Next ${post.cat[0]} Post</a>` : ""}
                `;

                let content;
                if (post.book) {
                    document.head.innerHTML += /*html*/ `<link rel="stylesheet" href="../style/book.css">`;
                    // pull the actual content from the crowbook export
                    let html = new JSDOM(file);
                    let page = html.window.document.getElementById("page");
                    let header = page.getElementsByTagName("header");
                    while (header.length > 0)
                        header[0].parentNode.removeChild(header[0]);
                    content = /*html*/ `
                        ${post.prompt ? /*html*/ `<p class="meta-info"><i>This story was inspired by a Reedsy Prompt and submitted to their competition. As such, it has also been published on <a href="${post.prompt}">their website</a>.</i></p>` : ""}
                        ${page.outerHTML}
                    `;
                } else {
                    content = converter.makeHtml(file.toString());
                }

                document.getElementById("main").innerHTML = /*html*/ `
                    <div class="list-display rounded">
                        <div class="blog-isolated">
                            <h1>${post.name}</h1>
                            ${post.archived ? /*html*/ `<p class="meta-info"><i>This post has been archived.</i></p>` : ""}
                            ${content}
                            <span class="text-muted project-status blog-isolated-status">${post.date}</span>
                            ${post.discuss ? /*html*/ `<a href="${post.discuss}" class="blog-discuss" id="blog-discuss-${post.id}">Discuss this post</a>` : ""}
                        </div>
                    </div>
                `;

                let ret = dom.serialize();
                fs.mkdir(`${folder}blog`, function () {
                    fs.writeFile(`${folder}blog/${post.id}.html`, ret, function () {});
                });
            });
        }
    });
});

function getAdjacentPost(json, index, move) {
    let cat = json[index].cat[0];
    while (true) {
        index += move;
        let post = json[index];
        if (!post)
            break;
        // only link to posts with the same main category
        if (post.cat[0] === cat && !post.archived)
            return post;
    }
}